@@ -2,7 +2,6 @@ from pathlib import Path
 from redis import Redis
 
 from flask import Flask, redirect, send_from_directory
-from flask_cors import CORS
 
 import json
 from os import getenv
@@ -23,10 +22,6 @@ def create_app(test_config: dict = None) -> Flask:
 
     app = Flask(__name__, instance_relative_config=True)
     app.config.from_mapping(
-        CA_PUBKEY=None,
-        STORE_PATH=app.instance_path + "/public/store",
-        JSON_PATH=app.instance_path + "/public/json",
-        CACHE_PATH=app.instance_path + "/cache/",
         REDIS_CONN=Redis(host=redis_host, port=redis_port),
         TESTING=False,
         DEBUG=False,
@@ -37,7 +32,6 @@ def create_app(test_config: dict = None) -> Flask:
     if not test_config:
         for config_file in [
             Path.cwd() / "config.py",
-            app.instance_path + "/config.py",
             "/etc/asu/config.py",
         ]:
             if Path(config_file).exists():
@@ -48,28 +42,21 @@ def create_app(test_config: dict = None) -> Flask:
         app.config.from_mapping(test_config)
 
     for option, value in app.config.items():
-        if option.endswith("_PATH") and isinstance(value, str):
+        if option.endswith("_PATH") and isinstance(value, (Path, str)):
             app.config[option] = Path(value)
             app.config[option].mkdir(parents=True, exist_ok=True)
 
-    Path(app.instance_path).mkdir(exist_ok=True, parents=True)
+    (Path().cwd()).mkdir(exist_ok=True, parents=True)
 
-    CORS(app, resources={r"/api/*": {"origins": "*"}})
+    @app.route("/json/")
+    @app.route("/json/<path:path>")
+    def json_path(path="index.html"):
+        return send_from_directory(app.config["JSON_PATH"], path)
 
-    # only serve files in DEBUG/TESTING mode
-    # production should use nginx for static files
-    if app.config["DEBUG"] or app.config["TESTING"]:
-
-        @app.route("/")
-        @app.route("/<path:path>")
-        def root(path="index.html"):
-            return send_from_directory(Path(app.instance_path) / "public", path)
-
-    else:
-
-        @app.route("/")
-        def root(path="index.html"):
-            return redirect("https://github.com/aparcar/asu/#api")
+    @app.route("/store/")
+    @app.route("/store/<path:path>")
+    def store_path(path="index.html"):
+        return send_from_directory(app.config["STORE_PATH"], path)
 
     from . import janitor
 
