@@ -74,7 +74,7 @@ def build(req: dict):
     job = get_current_job()
 
     log.debug(f"Building {req}")
-    cache = (req["cache_path"] / req["version"] / req["target"]).parent
+    cache = (Path.cwd() / "cache" / req["version"] / req["target"]).parent
     target, subtarget = req["target"].split("/")
     sums_file = Path(cache / f"{subtarget}_sums")
     sig_file = Path(cache / f"{subtarget}_sums.sig")
@@ -148,20 +148,13 @@ def build(req: dict):
         repos_path.write_text(repos)
         log.debug(f"Repos:\n{repos}")
 
-        # if a CA pubkey is defined, add it to the created images
-        # enabling CA requires each worker to find a seckey, pubkey and newcert
-        if req.get("ca_pubkey"):
-            fingerprint = fingerprint_pubkey_usign(req["ca_pubkey"])
-            opkg_key_path = cache / subtarget / "files/etc/opkg/keys"
-            opkg_key_path.mkdir(parents=True, exist_ok=True)
-            (opkg_key_path / fingerprint).write_text(
-                f"untrusted comment: ASU CA pubkey {fingerprint}\n{req['ca_pubkey']}"
-            )
-
+        if (Path.cwd() / "seckey").exists():
             # link key-build to imagebuilder
             (cache / subtarget / "key-build").symlink_to(Path.cwd() / "seckey")
+        if (Path.cwd() / "pubkey").exists():
             # link key-build.pub to imagebuilder
             (cache / subtarget / "key-build.pub").symlink_to(Path.cwd() / "pubkey")
+        if (Path.cwd() / "newcert").exists():
             # link key-build.ucert to imagebuilder
             (cache / subtarget / "key-build.ucert").symlink_to(Path.cwd() / "newcert")
 
@@ -249,7 +242,7 @@ def build(req: dict):
             "make",
             "manifest",
             f"PROFILE={req['profile']}",
-            f"PACKAGES={' '.join(req['packages'])}",
+            f"PACKAGES={' '.join(req.get('packages', ''))}",
             "STRIP_ABI=1",
         ],
         text=True,
