@@ -42,24 +42,40 @@ def get_file_hash(path: str) -> str:
     return h.hexdigest()
 
 
-def get_request_hash(request_data: dict) -> str:
+def get_manifest_hash(manifest: dict) -> str:
+    """Return sha256sum of package manifest
+
+    Duplicate packages are automatically removed and the list is sorted to be
+    reproducible
+
+    Args:
+        manifest(dict): list of packages
+
+    Returns:
+        str: hash of `req`
+    """
+    return str(hash(frozenset(sorted(manifest.items()))))
+
+
+def get_request_hash(req: dict) -> str:
     """Return sha256sum of an image request
 
     Creates a reproducible hash of the request by sorting the arguments
 
     Args:
-        request_data (dict): dict contianing request information
+        req (dict): dict contianing request information
 
     Returns:
-        str: hash of `request_data`
+        str: hash of `req`
     """
-    request_data["packages_hash"] = get_packages_hash(request_data.get("packages", ""))
     request_array = [
-        request_data.get("distro", ""),
-        request_data.get("version", ""),
-        request_data.get("profile", "").replace(",", "_"),
-        request_data["packages_hash"],
-        str(request_data.get("diff_packages", False)),
+        req.get("distro", ""),
+        req.get("version", ""),
+        req.get("version_code", ""),
+        req.get("profile", "").replace(",", "_"),
+        get_packages_hash(req.get("packages", "")),
+        get_manifest_hash(req.get("packages_versions", {})),
+        str(req.get("diff_packages", False)),
     ]
     return get_str_hash(" ".join(request_array), 12)
 
@@ -71,14 +87,27 @@ def get_packages_hash(packages: list) -> str:
     reproducible
 
     Args:
-        packages (list ): list of packages
+        packages (list): list of packages
 
     Returns:
-        str: hash of `request_data`
+        str: hash of `req`
     """
     return get_str_hash(" ".join(sorted(list(set(packages)))), 12)
 
 
+def fingerprint_pubkey_usign(pubkey: str) -> str:
+    """Return fingerprint of signify/usign public key
+
+    Args:
+        pubkey (str): signify/usign public key
+
+    Returns:
+        str: string containing the fingerprint
+    """
+    keynum = base64.b64decode(pubkey.splitlines()[-1])[2:10]
+    return "".join(format(x, "02x") for x in keynum)
+
+
 def verify_usign(sig_file: Path, msg_file: Path, pub_key: str) -> bool:
     """Verify a signify/usign signature
 
