@@ -14,6 +14,7 @@ def get_redis():
 
 
 def parse_packages_file(url, repo):
+    r = get_redis()
     req = requests.get(url)
 
     if req.status_code != 200:
@@ -21,24 +22,33 @@ def parse_packages_file(url, repo):
         return {}
 
     packages = {}
+    mapping = {}
     linebuffer = ""
     for line in req.text.splitlines():
         if line == "":
             parser = email.parser.Parser()
             package = parser.parsestr(linebuffer)
-            package_name = package.get("SourceName")
-            if package_name:
-                packages[package_name] = dict(
+            source_name = package.get("SourceName")
+            if source_name:
+                packages[source_name] = dict(
                     (name.lower().replace("-", "_"), val)
                     for name, val in package.items()
                 )
-                packages[package_name]["repository"] = repo
+                packages[source_name]["repository"] = repo
+                package_name = package.get("Package")
+                if source_name != package_name:
+                    mapping[package_name] = source_name
             else:
                 print(f"Something wired about {package}")
             linebuffer = ""
         else:
             linebuffer += line + "\n"
 
+    for package, source in mapping.items():
+        if not r.hexists("mapping-abi", package):
+            print(f"Add ABI mapping {package} -> {source}")
+            r.hset("mapping-abi", package, source)
+
     current_app.logger.debug(f"Found {len(packages)} in {repo}")
 
     return packages
@@ -78,10 +88,12 @@ def update_branch(branch):
 
     r.sadd(f"targets-{branch['name']}", *list(targets))
 
-    architectures = set(branch["targets"].values())
+    packages_path = branch["path_packages"].format(branch=branch["name"])
 
     with Pool(20) as pool:
-        pool.starmap(update_arch_packages, map(lambda a: (branch, a), architectures))
+        pool.starmap(
+            update_arch_packages, map(lambda a: (branch, a), branch["targets"].values())
+        )
 
     for version in branch["versions"]:
         current_app.logger.info(f"Update {branch['name']}/{version}")
@@ -89,7 +101,6 @@ def update_branch(branch):
             # TODO: ugly
             version_path = branch["path"].format(version=version)
             version_path_abs = current_app.config["JSON_PATH"] / version_path
-            packages_path = branch["path_packages"].format(branch=branch["name"])
             output_path = current_app.config["JSON_PATH"] / packages_path
             version_path_abs.mkdir(exist_ok=True, parents=True)
             packages_symlink = version_path_abs / "packages"
@@ -133,7 +144,32 @@ def update_target_packages(branch: dict, version: str, target: str):
     version_path = branch["path"].format(version=version)
     r = get_redis()
 
-    packages = get_packages_target_base(branch, branch["versions"][0], target)
+    packages_modified_local = r.get(
+        f"last-modified-packages-{branch['name']}-{version}-{target}"
+    )
+    if packages_modified_local:
+        packages_modified_local = packages_modified_local.decode("utf-8")
+    packages_modified_remote = requests.head(
+        current_app.config["UPSTREAM_URL"]
+        + "/"
+        + version_path
+        + f"/targets/{target}/packages/Packages.manifest",
+    ).headers.get("last-modified")
+
+    if packages_modified_local:
+        if packages_modified_local == packages_modified_remote:
+            current_app.logger.debug(
+                f"Skip {branch['name']}/{version}/{target} package update"
+            )
+            return
+
+    if packages_modified_remote:
+        r.set(
+            f"last-modified-packages-{branch['name']}-{version}-{target}",
+            packages_modified_remote,
+        )
+
+    packages = get_packages_target_base(branch, version, target)
 
     if len(packages) == 0:
         current_app.logger.warning(f"No packages found for {target}")
@@ -168,8 +204,26 @@ def update_target_packages(branch: dict, version: str, target: str):
 def update_arch_packages(branch: dict, arch: str):
     current_app.logger.info(f"Update {branch['name']}/{arch}")
     r = get_redis()
-    packages = {}
+
     packages_path = branch["path_packages"].format(branch=branch["name"])
+    packages_modified_local = r.get(f"last-modified-packages-{branch['name']}-{arch}")
+    if packages_modified_local:
+        packages_modified_local = packages_modified_local.decode("utf-8")
+    packages_modified_remote = requests.head(
+        current_app.config["UPSTREAM_URL"] + f"/{packages_path}/{arch}/feeds.conf"
+    ).headers.get("last-modified")
+
+    if packages_modified_local:
+        if packages_modified_local == packages_modified_remote:
+            current_app.logger.debug(f"Skip {branch['name']}/{arch} package update")
+            return
+
+    if packages_modified_remote:
+        r.set(
+            f"last-modified-packages-{branch['name']}-{arch}", packages_modified_remote
+        )
+
+    packages = {}
 
     # first update extra repos in case they contain redundant packages to core
     for name, url in branch.get("extra_repos", {}).items():
@@ -216,10 +270,32 @@ def update_target_profiles(branch: dict, version: str, target: str):
         current_app.config["UPSTREAM_URL"]
         + f"/{version_path}/targets/{target}/profiles.json"
     )
+
     if req.status_code != 200:
-        current_app.logger.warning(f"Could not download profiles.json for {target}")
+        current_app.logger.warning(
+            f"Could not download profiles.json for {version}/{target}"
+        )
         return
 
+    profiles_modified_local = r.get(
+        f"last-modified-profiles-{branch['name']}-{version}-{target}"
+    )
+    if profiles_modified_local:
+        profiles_modified_local = profiles_modified_local.decode("utf-8")
+
+    profiles_modified_remote = req.headers.get("last-modified")
+
+    if profiles_modified_local:
+        if profiles_modified_local == profiles_modified_remote:
+            current_app.logger.debug(f"Skip {branch['name']}/{version} profiles update")
+            return
+
+    if profiles_modified_remote:
+        r.set(
+            f"last-modified-profiles-{branch['name']}-{version}-{target}",
+            profiles_modified_remote,
+        )
+
     req = requests.get(
         current_app.config["UPSTREAM_URL"]
         + f"/{version_path}/targets/{target}/profiles.json"
@@ -233,7 +309,7 @@ def update_target_profiles(branch: dict, version: str, target: str):
     for profile, data in profiles.items():
         for supported in data.get("supported_devices", []):
             r.hset(f"mapping-{branch['name']}-{version}", supported, profile)
-        r.hset(f"profiles-{branch['name']}-{version}", profile, target)
+        r.sadd(f"profiles-{branch['name']}-{version}-{target}", profile)
         profile_path = (
             current_app.config["JSON_PATH"]
             / version_path
